% rlocus - creates a root locus plot
% 
% USAGE
%
% [A K] = rlocus(N, D)
% [A K] = rlocus(N, D, k) returns graphs the root locus plot of the open 
% loop transfer function where N is a vector of coefficients for the 
% numerator and D is a vector of coefficents for the denominator and 
% k is a vector of gains.  If k is not specified, a best guess vector
% of gains is created.
%
% Returns 
%
% A is a matrix containing each root path
% K is a matrix containing the gain at each point in the root path
% 
% For the transfer function H = (s + 1) / (s^2 + s + 1) evaluated between 1 
% and 1000 radians per second then the root locus can be given by
% 
% N = [0 1 1];
% D = [1 1 1];
% k = 0:0.1:5;
% rlocus(N,D,k);
%
% Licensed under the GPL
% Copyright (C) 2014 Jonathan Weaver <jonw0224@aim.com>

function [A K] = rlocus(N, D, k)

% This file is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


    p = roots(D);                                       %Poles
    ze = roots(N);                                      %Zeros
    pex = length(p) - length(ze);                       %Pole Zero Excess for asymptotes
    r = 10.*max([abs(p); abs(ze)]);                     %For automatic zoom
    hold on;                                            %Plot the imaginary axis
    plot([0 0],[-r r],'k:');
    
    % Ensure the lengths of N and D are equal
    if(length(N) < length(D))                           
        N = [zeros(1, length(D) - length(N)) N];
    else
        D = [zeros(1, length(N) - length(D)) D];
    end

    %Calculate the centroid and asymptotes
    centroid = (sum(real(p)) - sum(real(ze)))./pex;
    angles = pi.*(1 + 2.*(0:1:(pex-1)))./pex;
    
    asymx = [zeros(1,length(angles)); r*cos(angles)] + centroid;
    asymy = [zeros(1,length(angles)); r*sin(angles)];

    %Plot the asymptotes
    for m = 1:length(angles)
        plot(asymx(:,m), asymy(:,m),'y.--');
    end
    
    %Plot the centroid
    plot(centroid,0,'ys');   
    
    %Plot the poles and zeros
    plot(real(p), imag(p), 'bx');
    plot(real(ze), imag(ze), 'ro');
       
    % if k is unspecified, automatically scale
    if(nargin == 2)
        % Initial guess
        kend = prod(abs(p)+1)./(max(abs(N))+1)./(max(abs(D))+1);
        done = 0;
        while(done == 0)
            c = D+kend*N;
            er = roots(c);
            if(pex>0)
                if(max(abs(er)) > r / 5)
                    done = 1;
                else
                    kend = kend*2;
                end
            else
                delta = sort(er) - sort(ze);
                if(max(abs(delta))>r/5/10)
                    kend = kend*2;
                else
                    done = 1;
                end
            end
        end
        k = 0:kend/2000:kend;        
    end

    %Set output gain variable
    K = k;

    %Calculate the paths of the poles
    C = ones(size(k'))*D + k'*N;
    B = size(C);
    A = zeros(B(1), B(2)-1);
    for m = 1:B(1)
        A(m,:) = roots(C(m,:));
    end
    
    %Define an index variable
    n = 1:size(A)(1)-1;

    % If k is unspecified, get the break in and break out points drawn a little better
    if(nargin == 2)
        cnt = 1;
        bk = sum(abs(imag(A))<r/1000000,2);
        ind = find(bk(n+1)-bk(n));
        if(length(ind) > 0)
            ubk = k(ind+1);
            lbk = k(ind);
            while(max(ubk-lbk) > kend/200000)
                k(ind+1) = (ubk+lbk)./2;
                C(ind+1,:) = ones(size(ind))*D + (k(ind+1))*N;
                for m = 1:length(ind)
                    A(ind(m)+1,:) = roots(C(ind(m)+1,:));
                end
                tst = max(((r/10000 < abs(imag(A(ind+1,:)))).*abs(imag(A(ind+1,:)))>0)')';
                tst2 = max(((r/10000 < abs(imag(A(ind,:)))).*abs(imag(A(ind,:)))>0)')';
                tst = (tst & ~tst2) | (~tst & tst2);
                ubk = tst.*k(ind+1) + ~tst.*ubk;
                lbk = tst.*lbk + ~tst.*k(ind+1);
            end
        end
    end
    
    % Ensure the order of the roots is consistent for each path using trianglulation
    % Calculate using first point, sort roots based on distance and angle from r+jr
    [b in] = sort(A+r+j*r,2);
    At = A';
    A = At(size(A)(2)*(0:(size(A)(1)-1))'*ones(1,size(A)(2))+in);

    % Look for swaps in the path
    b = abs(A(n,:)-A(n+1,:));
    delta = b>(mean(mean(b))+20*std(std(b)));
    rowswap = find(sum(delta,2)>0);
    % If there are swaps and swaps count less than the number of roots
    if(length(rowswap) > 0 & length(rowswap) < length(p))
        % Perform swaps based on a sort based on distance and angle from -r+j0
        for m = 1:length(rowswap)
            [b in] = sort(A(rowswap(m),:)-r);
            [b in2] = sort(A(rowswap(m)+1,:)-r);
            A(rowswap(m)+1:end,in) = A(rowswap(m)+1:end,in2);
        end
    end
        
    %Plot the paths
    for m = 1:(B(2)-1)
        plot(real(A(:,m)), imag(A(:,m)),'g');
    end
    hold off;
    
    %Autozoom
    xlim([-r/5,r/5]);
    ylim([-r/5,r/5]);
end
